#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
} vert;

in vec3 Normal;
in vec3 FragPos;
in vec2 TexCoord;
in vec4 FragPosLightSpace;

uniform vec3 camera_world;
uniform bool tex = false;

//Material
uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform vec3 diffuseColor = vec3(1, 1, 1);

//Lights
uniform struct DirLight {
    vec3 direction;
	vec3 color;
} dirLight;


//Textures
uniform sampler2D texture_diffuse0;
uniform sampler2D texture_specular0;

uniform sampler2D shadowMap;

uniform float illuminationMultiplier;

out vec4 color;

vec4 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, vec4 specMap, vec4 diffMap);
vec4 celShading(vec4 color);

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // check whether current frag pos is in shadow
	float bias = max(0.00 * (1.0 - dot(Normal, dirLight.direction)), 0.000);  
	//bias = 0.0005;
    float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
		    float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
		    shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
		}    
	}
	shadow /= 9.0;
	//shadow = 0.0f;

    return shadow;
}  

void main() {	
	vec3 norm = normalize(Normal);
	vec3 viewDir = normalize(camera_world - FragPos);
	vec4 specMap = vec4(1.0f, 1.0f, 1.0f, 1.0f);
	vec4 diffMap = vec4(diffuseColor, 1.0);

	if(tex){
		color = vec4(1, 1, 0, 1);
		diffMap = texture(texture_diffuse0, TexCoord);
		specMap = texture(texture_specular0, TexCoord);
	}

	vec4 result = CalcDirLight(dirLight, norm, viewDir, specMap, diffMap);
	float shadow = ShadowCalculation(FragPosLightSpace);
	result *= (1.0 - shadow);
	//vec4 result = vec4(dirLight.color, 1) * diffMap;
	//result += vec4(dirLight.color, 1) * specMap;
	
	//vec3 ambient = materialCoefficients.x * diffuseColor;
	vec4 ambient = vec4(materialCoefficients.x * diffuseColor, 1);
	if(tex){
		ambient = materialCoefficients.x * diffMap;
	}
	result += vec4(ambient.rgb, 1);
	color = vec4(result.rgb * illuminationMultiplier, result.a);
	color = celShading(color);
	//color = diffMap + specMap;
	//color = vec4(1, 0, 0, 1);
}

vec4 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, vec4 specMap, vec4 diffMap)
{
    vec3 lightDir = normalize(-light.direction);
    // diffuse shading
    float lambert = max(dot(normal, lightDir), 0.0);
	vec4 diffuse = vec4(light.color, 1) * materialCoefficients.y * lambert * diffMap;
	//diffuse = celShading(diffuse);
	// specular shading
	vec3 reflectDir = reflect(-lightDir, normal);
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), specularAlpha);
	//vec3 h = (lightDir + viewDir) / (abs(lightDir + viewDir));
	//float spec = pow(max(dot(normal, h), 0.0), specularAlpha);
	vec4 specular = vec4(light.color, 1) * materialCoefficients.z * spec * specMap;
	   
    //return (diffuse + specular); Auskommentiert da Cel Shading sonst weird aussieht 
	return diffuse;
}

vec4 celShading(vec4 color){
	//Calculate RGB to HSV and change the V (Brightness value)
	float H, S, V;
	float Cmax = max(max(color.x, color.y), color.z);
	float Cmin = min(min(color.x, color.y), color.z);
	float delta = Cmax - Cmin;
	//Calculate V
	V = Cmax;
	//Calculate S
	if(Cmax == 0){
		S = 0;
	}else{
		S = delta / Cmax;
	}
	//Calculate H
	if(delta == 0){
		H = 0;
	}else if(Cmax == color.x){
		H = 0.6f * (mod(((color.y - color.z)/delta), 6.0f));
	}else if(Cmax == color.y){
		H = 0.6f * (((color.z - color.x)/delta)+2);
	}else if(Cmax == color.z){
		H = 0.6f * (((color.x - color.y)/delta)+4);
	}
	H = mod(H, 3.6f);
	H = H * 100;

	//Change V value
	float colorLevels = 4;

	float level = floor(V * colorLevels);
	V = level / colorLevels;

	//Now calculate back to RGB
	float R, G, B;

	float C = V * S;
	float X = C * (1 - abs(mod((H/60),2.0f) - 1));
	float m = V - C;

	if(H >= 0 && H < 60){
		R = C;
		G = X;
		B = 0;
	}else if(H >= 60 && H < 120){
		R = X;
		G = C;
		B = 0;
	}else if(H >= 120 && H < 180){
		R = 0;
		G = C;
		B = X;
	}else if(H >= 180 && H < 240){
		R = 0;
		G = X;
		B = C;
	}else if(H >= 240 && H < 300){
		R = X;
		G = 0;
		B = C;
	}else if(H >= 300 && H < 360){
		R = C;
		G = 0;
		B = X;
	}
	R = R + m;
	G = G + m;
	B = B + m;
	return vec4(R, G, B, color.a);
}
