#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
} vert;

in vec3 Normal;
in vec3 FragPos;
in vec2 TexCoord;
in vec4 FragPosLightSpace;

uniform vec3 camera_world;
uniform bool tex = false;

//Material
uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform vec3 diffuseColor = vec3(1, 1, 1);

//Lights
uniform struct DirLight {
    vec3 direction;
	vec3 color;
} dirLight;


//Textures
uniform sampler2D texture_diffuse0;
uniform sampler2D texture_specular0;

uniform sampler2D shadowMap;
uniform samplerCube cubemap;


uniform float illuminationMultiplier;


out vec4 color;

vec4 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, vec4 specMap, vec4 diffMap);

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // check whether current frag pos is in shadow
	float bias = max(0.001 * (1.0 - dot(Normal, dirLight.direction)), 0.0003);  
	//bias = 0.0005;
    float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	for(int x = -1; x <= 1; ++x)
	{
		for(int y = -1; y <= 1; ++y)
		{
		    float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
		    shadow += currentDepth - bias > pcfDepth ? 1.0 : 0.0;        
		}    
	}
	shadow /= 9.0;
	//shadow = 0.0f;

    return shadow;
}  

void main() {	
	vec3 norm = normalize(Normal);
	vec3 viewDir = normalize(camera_world - FragPos);
	vec4 specMap = vec4(1.0f, 1.0f, 1.0f, 1.0f);
	vec4 diffMap = vec4(diffuseColor, 1.0);

	//Environmental Mapping
	vec3 I = normalize(FragPos - camera_world);
    vec3 R = reflect(I, normalize(Normal));

	if(tex){
		color = vec4(1, 1, 0, 1);
		diffMap = texture(texture_diffuse0, TexCoord);
		specMap = texture(texture_specular0, TexCoord);
	}

	vec4 result = CalcDirLight(dirLight, norm, viewDir, specMap, diffMap);
	float shadow = ShadowCalculation(FragPosLightSpace);
	result *= (1.0 - shadow);
	//vec4 result = vec4(dirLight.color, 1) * diffMap;
	//result += vec4(dirLight.color, 1) * specMap;
	
	//vec3 ambient = materialCoefficients.x * diffuseColor;
	vec4 ambient = vec4(materialCoefficients.x * diffuseColor, 1);
	if(tex){
		ambient = materialCoefficients.x * diffMap;
	}
	result += vec4(ambient.rgb, 1);
	vec4 skyBox = vec4(texture(cubemap, R).rgb, 1.0);
	color = vec4(result.rgb * illuminationMultiplier, result.a);
	color = mix(color, skyBox, 0.1);
	//color = diffMap + specMap;
	//color = vec4(1, 0, 0, 1);
}

vec4 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir, vec4 specMap, vec4 diffMap)
{
    vec3 lightDir = normalize(-light.direction);
    // diffuse shading
    float lambert = max(dot(normal, lightDir), 0.0);
	vec4 diffuse = vec4(light.color, 1) * materialCoefficients.y * lambert * diffMap;

	// specular shading
	vec3 reflectDir = reflect(-lightDir, normal);
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), specularAlpha);
	//vec3 h = (lightDir + viewDir) / (abs(lightDir + viewDir));
	//float spec = pow(max(dot(normal, h), 0.0), specularAlpha);
	vec4 specular = vec4(light.color, 1) * materialCoefficients.z * spec * specMap;
	   
    return (diffuse+specular);
	//return ambient;
}
